#!/usr/bin/env python

"""
    barrister
    ~~~~~~~~~

    A RPC toolkit for building lightweight reliable services.  Ideal for
    both static and dynamic languages.

    :copyright: (c) 2012 by James Cooper.
    :license: MIT, see LICENSE for more details.
"""

import time
import unittest
from barrister.parser import parse, IdlParseException

def ret_field(type_, is_array=False, optional=False):
    return { "type": type_, "is_array": is_array, "optional": optional }

def field(name, type_, comment="", is_array=False, optional=False):
    return { "type": type_, "name": name, "comment": comment, "is_array": is_array, "optional":  optional}

class ParserTest(unittest.TestCase):

    def test_parse_comment(self):
        idl = """
// # section
// foo

// this is a person
struct Person {
   age int
}"""
        expected = [ { "type" : "comment", "value" : "# section\nfoo" },
                     { "type" : "struct", "name" : "Person", "extends" : "",
                       "comment" : "this is a person", "fields" : [
                    field("age", "int") ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))

    def test_parse_struct(self):
        idl = """struct Person {
email string
age int
}"""
        expected = [ { "name" : "Person", 
                       "type" : "struct", 
                       "comment" : "", "extends" : "",
                       "fields" : [ field("email", "string"), field("age", "int") ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))

    def test_parse_multiple(self):
        idl = """struct Person { email string } 
struct Animal { furry bool }"""
        expected = [ { "name" : "Person", 
                       "type" : "struct", 
                       "comment" : "", "extends" : "",
                       "fields" : [ field("email", "string") ] },
                     { "name" : "Animal", 
                       "type" : "struct", 
                       "comment" : "", "extends" : "",
                       "fields" : [ field("furry", "bool") ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))

    def test_parse_enum(self):
        idl = """enum Status { success fail
invalid }"""
        expected = [ { "name" : "Status", "type" : "enum", "comment" : "",
                       "values" : [ { "value" : "success", "comment" : "" },
                                    { "value" : "fail", "comment" : "" },
                                    { "value" : "invalid", "comment" : "" } ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))

    def test_parse_interface(self):
        idl = """interface MyService {
    add(a int, b int) int
    login(req LoginRequest) LoginResponse
}
"""
        expected = [ { "name" : "MyService", 
                       "type" : "interface",
                       "comment" : "",
                       "functions" : [
                    { "name" : "add", "comment" : "", "returns" : ret_field("int"), 
                      "params" : [
                            { "type" : "int", "name" : "a", "is_array": False },
                            { "type" : "int", "name" : "b", "is_array": False } ] },
                    { "name" : "login", "comment" : "",
                      "returns" : ret_field("LoginResponse"), "params" : [
                            { "type" : "LoginRequest", "name" : "req", "is_array": False } ] } ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False, validate=False))

    def test_invalid_struct(self):
        idl = """struct foo { """
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 1, "message" : "Unexpected end of file" } ]
            self.assertEquals(expected, e.errors)

    def test_missing_name(self):
        idls = [ "struct  {", "enum {", "interface { " ]
        for idl in idls:
            try:
                parse(idl, add_meta=False)
                self.fail("should have thrown exception")
            except IdlParseException as e:
                expected = [ { "line": 1, "message" : "Missing identifier" } ]
                self.assertEquals(expected, e.errors)

    def test_enum_comments(self):
        idl = """enum Status {
     // Request successful
     success }"""
        expected = [ { "name" : "Status", "type" : "enum", "comment" : "",
                       "values" : [ { "comment" : "Request successful",
                                      "value": "success" } ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))        

    def test_array_type(self):
        idl = """struct Animal  {
    friend_names []string }"""
        expected = [ { "name" : "Animal", "type" : "struct", "comment" : "",
                       "extends" : "",
                 "fields" : [ field("friend_names", "string", "", True) ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))

    def test_array_return_type(self):
        idl = """interface FooService {
    repeat(s string) []string
}"""
        expected = [ { "name" : "FooService", "type" : "interface", "comment" : "",
                       "functions" : [
                    { "name" : "repeat",  "comment" : "",
                      "returns" : ret_field("string", True),
                      "params" : [ { "type" : "string", "name" : "s", "is_array": False } ] } ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))

    def test_struct_comments(self):
        idl = """struct Animal   {
     // fur color
     color string }"""
        expected = [ { "name" : "Animal", "type" : "struct", "comment" : "",
                       "extends" : "",
                       "fields" : [ field("color", "string", "fur color") ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))        

    def test_function_comments(self):
        idl = """interface FooService {
     //Add two numbers
     // a is the 1st num
     //  b is the 2nd num
     add(a int, b []int) int
}"""
        expected = [ { "name" : "FooService", "type" : "interface", "comment" : "",
                       "functions" : [
                    { "name" : "add", "returns" : ret_field("int"), 
                      "comment" : "Add two numbers\na is the 1st num\n b is the 2nd num",
                      "params" : [ 
                            { "type" : "int", "name" : "a", "is_array": False },
                            { "type" : "int", "name" : "b", "is_array": True } ] } ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))        

    def test_interface_comments(self):
        idl = """// FooService is a..
// and does other stuff
interface FooService {
    blah99() blah_Response
}"""
        expected = [ { "name" : "FooService", "type" : "interface",
                       "comment" : "FooService is a..\nand does other stuff",
                       "functions" : [
                    { "name" : "blah99", "returns" : ret_field("blah_Response"),
                      "comment" : "", "params" : [ ] }
                    ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False, validate=False))
        

    def test_extends_struct(self):
        idl = """struct Animal {
   color string
   gender string
}

struct Cat extends Animal {
    purr_volume int
}"""
        expected = [ { "name" : "Animal", "type" : "struct", 
                       "extends" : "", "comment" : "",
                       "fields" : [ field("color", "string"), field("gender", "string") ] },
                     { "name" : "Cat", "type" : "struct", 
                       "extends" : "Animal", "comment" : "",
                       "fields" : [ field("purr_volume", "int") ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))

    def test_no_dupe_types(self):
        idl = """struct Animal {
    color string
}
enum Animal {
    foo
}
interface Foo {
    doSomething() bool
}
struct Foo {
    color string
}
enum Blarg {  stuff }
interface Blarg {
    do_other() bool
}"""
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 4, "message" : "type Animal already defined" },
                         { "line": 10, "message" : "type Foo already defined" },
                         { "line": 14, "message" : "type Blarg already defined" } ]
            self.assertEquals(expected, e.errors)

    def test_no_cycles(self):
        idl = """struct Animal {
    home Location
}
struct Location {
    residents []Animal
}"""
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 0, "message" : "cycle detected in: struct Animal" },
                         { "line": 0, "message" : "cycle detected in: struct Location" } ]
            self.assertEquals(expected, e.errors)

    def test_cycle_detection(self):
        idl = """struct Book {
    author string
}
struct TaskResult {
    toLoan []Book
    toAck  []Book
}"""
        parse(idl, add_meta=False)

    def test_interface_cant_be_field_type(self):
        idl = """struct Animal {
    svc FooService
}
interface FooService {
    do_something() bool
}"""
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 0, "message" : "interface FooService cannot be a field type" } ]
            self.assertEquals(expected, e.errors)        

    def test_types_exist(self):
        idl = """struct Animal {
    color Color
}
interface FooService {
    saySomething(cat Cat) Saying
}
struct Blarg extends Foo {
   a int
}"""
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 0, "message" : "undefined type: Color" },
                         { "line": 0, "message" : "undefined type: Cat" },
                         { "line": 0, "message" : "undefined type: Saying" },
                         { "line": 0, "message" : "undefined type: Foo" } ]
            self.assertEquals(expected, e.errors)

    def test_cant_override_parent_field(self):
        idl = """struct Animal {
    color string
    gender string
}
struct Cat    extends Animal {
   color int
}
struct Manx extends Cat {
   gender bool
}"""
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 0, "message" : "Cat cannot redefine parent field color" },
                         { "line": 0, "message" : "Manx cannot redefine parent field gender" } ]
            self.assertEquals(expected, e.errors)

    def test_struct_cant_extend_enum(self):
        idl = """enum Status { foo }
struct Animal extends Status {
    color string
}"""
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 0, "message" : "Animal cannot extend enum Status" } ]
            self.assertEquals(expected, e.errors)        

    def test_struct_cant_extend_native_type(self):
        idl = """struct Animal extends float {
    color string
}"""
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 0, "message" : "Animal cannot extend float" } ]
            self.assertEquals(expected, e.errors)        

    def test_struct_must_have_fields(self):
        idl = "struct Animal { }"
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 1, "message" : "Animal must have at least one field" } ]
            self.assertEquals(expected, e.errors)        

    def test_interface_must_have_funcs(self):
        idl = "interface FooService { }"
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 1, "message" : "FooService must have at least one function" } ]
            self.assertEquals(expected, e.errors)        

    def test_enum_must_have_values(self):
        idl = "enum Status { }"
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 1, "message" : "Status must have at least one value" } ]
            self.assertEquals(expected, e.errors)        

    def test_cycle_detection_for_interfaces(self):
        idl = """struct BaseResponse {
    status int
}
interface FooService {
    add(a int, b int) BaseResponse
    subtract(a int, b int) BaseResponse
}"""
        # should work - cycle detection should reset per function
        parse(idl, add_meta=False)

    def test_interface_cant_be_param(self):
        idl = """interface BlargService {
    do_stuff() int
}
interface FooService {
    add(a int, b BlargService) float
    subtract(a int, b int) BlargService
}"""
        try:
            parse(idl, add_meta=False)
            self.fail("should have thrown exception")
        except IdlParseException as e:
            expected = [ { "line": 0, "message" : "interface BlargService cannot be a field type" },
                         { "line": 0, "message" : "interface BlargService cannot be a field type" } ]
            self.assertEquals(expected, e.errors)        

    def test_optional_struct_field(self):
        idl = """struct Person {
   firstName string
   email string  [optional]
}"""
        expected = [ { "name" : "Person", "type" : "struct", 
                       "extends" : "", "comment" : "",
                       "fields" : [ field("firstName", "string"), 
                                    field("email", "string", optional=True) ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))

    def test_optional_return_type(self):
        idl = """interface FooService {
   sayHi() string [optional]
}"""
        expected = [ { "name": "FooService", "type": "interface", "comment": "",
                       "functions" : 
                       [ { "name" : "sayHi", "comment" : "", "params" : [ ],
                           "returns" : ret_field("string", optional=True) } ] } ]
        self.assertEquals(expected, parse(idl, add_meta=False))

    def test_add_meta(self):
        idl = """interface FooService {
    do_foo() string
}"""
        start = int(time.time() * 1000)
        parsed = parse(idl, add_meta=True)
        meta = parsed[-1]
        generated = meta["date_generated"]
        checksum = meta["checksum"]
        stop = int(time.time() * 1000)
        self.assertTrue(generated >= start and generated <= stop)
        self.assertTrue(checksum != None)

    def test_meta_checksum(self):
        base = "enum Y {\ndog\ncat\n}\nstruct Z {\n a int }\n"
        base2 = "// foo\nstruct Z {\n //foo2\na int }\nenum Y {\ncat\ndog\n}\n"
        equivalent = [ base+"interface FooService {\n  do_foo() string\n}",
                       "interface FooService {\n  do_foo() string\n}"+base2,
                       base+" interface  FooService  {\n  // stuff\n do_foo() string\n\n}" ]
        first_checksum = None
        for idl in equivalent:
            parsed = parse(idl, add_meta=True)
            meta = parsed[-1]
            checksum = meta["checksum"]
            if first_checksum == None:
                first_checksum = checksum
            else:
                self.assertEquals(first_checksum, checksum)

        base3 = "enum Y {\ndog2\ncat\n}\nstruct Z {\n a int }\n"
        base4 = "enum Y {\ndog\ncat\n}\nstruct Z {\n a float }\n"
        different = [ base3+"interface FooService {\n  do_foo() string\n}",
                      base4+"interface FooService {\n  do_foo() string\n}",
                      base+"interface FooService {\n  do_foo(a int) string\n}",
                      base+"interface FooService {\n  do_foo(a int) string\n do_bar() int\n}",
                      base+"// foo interface\n interface  FooService  {\n  // stuff\n do_foo() float\n\n}" ]
        for idl in different:
            parsed = parse(idl, add_meta=True)
            meta = parsed[-1]
            self.assertNotEquals(first_checksum, meta["checksum"])

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
    barrister
    ~~~~~~~~~

    A RPC toolkit for building lightweight reliable services.  Ideal for
    both static and dynamic languages.

    http://barrister.bitmechanic.com/

    :copyright: 2012 by James Cooper.
    :license: MIT, see LICENSE for more details.
"""
__version__ = '0.1.2'

from barrister.runtime import contract_from_file, idgen_uuid, idgen_seq
from barrister.runtime import RpcException, Server, Filter, HttpTransport, InProcTransport
from barrister.runtime import Client, Batch
from barrister.runtime import Contract, Interface, Enum, Struct, Function
from barrister.docco import docco_html



"""
    barrister
    ~~~~~~~~~

    A RPC toolkit for building lightweight reliable services.  Ideal for
    both static and dynamic languages.

    :copyright: (c) 2012 by James Cooper.
    :license: MIT, see LICENSE for more details.
"""
import unittest

from .runtime_test import RuntimeTest
from .parser_test import ParserTest

def all_tests():
    s = [ ]
    s.append(unittest.TestLoader().loadTestsFromTestCase(ParserTest))
    s.append(unittest.TestLoader().loadTestsFromTestCase(RuntimeTest))
    return unittest.TestSuite(s)



#!/usr/bin/env python

"""
    Module for generating Docco style HTML output based on a Barrister IDL file.

    See: http://jashkenas.github.com/docco/

    :copyright: 2012 by James Cooper.  Embedded CSS is copyright Jeremy Ashkenas.
    :license: MIT, see LICENSE for more details.
"""

import sys
import markdown
import optparse
try:
    import json
except:
    import simplejson as json

#
# Thank you Jeremy Ashkenas
# http://jashkenas.github.com/docco/
#
DOCCO_CSS="""
/*--------------------- Layout and Typography ----------------------------*/
body {
  font-family: 'Palatino Linotype', 'Book Antiqua', Palatino, FreeSerif, serif;
  font-size: 15px;
  line-height: 22px;
  color: #252519;
  margin: 0; padding: 0;
}
a {
  color: #261a3b;
}
  a:visited {
    color: #261a3b;
  }
p {
  margin: 0 0 15px 0;
}
h1, h2, h3, h4, h5, h6 {
  margin: 0px 0 15px 0;
}
  h1 {
    margin-top: 40px;
  }
#container {
  position: relative;
}
#background {
  position: fixed;
  top: 0; left: 525px; right: 0; bottom: 0;
  background: #f5f5ff;
  border-left: 1px solid #e5e5ee;
  z-index: -1;
}
#jump_to, #jump_page {
  background: white;
  -webkit-box-shadow: 0 0 25px #777; -moz-box-shadow: 0 0 25px #777;
  -webkit-border-bottom-left-radius: 5px; -moz-border-radius-bottomleft: 5px;
  font: 10px Arial;
  text-transform: uppercase;
  cursor: pointer;
  text-align: right;
}
#jump_to, #jump_wrapper {
  position: fixed;
  right: 0; top: 0;
  padding: 5px 10px;
}
  #jump_wrapper {
    padding: 0;
    display: none;
  }
    #jump_to:hover #jump_wrapper {
      display: block;
    }
    #jump_page {
      padding: 5px 0 3px;
      margin: 0 0 25px 25px;
    }
      #jump_page .source {
        display: block;
        padding: 5px 10px;
        text-decoration: none;
        border-top: 1px solid #eee;
      }
        #jump_page .source:hover {
          background: #f5f5ff;
        }
        #jump_page .source:first-child {
        }
table td {
  border: 0;
  outline: 0;
}
  td.docs, th.docs {
    max-width: 450px;
    min-width: 450px;
    min-height: 5px;
    padding: 10px 25px 1px 50px;
    overflow-x: hidden;
    vertical-align: top;
    text-align: left;
  }
    .docs pre {
      margin: 15px 0 15px;
      padding-left: 15px;
    }
    .docs p tt, .docs p code {
      background: #f8f8ff;
      border: 1px solid #dedede;
      font-size: 12px;
      padding: 0 0.2em;
    }
    .pilwrap {
      position: relative;
    }
      .pilcrow {
        font: 12px Arial;
        text-decoration: none;
        color: #454545;
        position: absolute;
        top: 3px; left: -20px;
        padding: 1px 2px;
        opacity: 0;
        -webkit-transition: opacity 0.2s linear;
      }
        td.docs:hover .pilcrow {
          opacity: 1;
        }
  td.code, th.code {
    padding: 14px 15px 16px 25px;
    width: 100%;
    vertical-align: top;
    background: #f5f5ff;
    border-left: 1px solid #e5e5ee;
  }
    pre, tt, code {
      font-size: 12px; line-height: 18px;
      font-family: Menlo, Monaco, Consolas, "Lucida Console", monospace;
      margin: 0; padding: 0;
    }


/*---------------------- Syntax Highlighting -----------------------------*/
td.linenos { background-color: #f0f0f0; padding-right: 10px; }
span.lineno { background-color: #f0f0f0; padding: 0 5px 0 5px; }
body .hll { background-color: #ffffcc }
body .c { color: #408080; font-style: italic }  /* Comment */
body .err { border: 1px solid #FF0000 }         /* Error */
body .k { color: #954121 }                      /* Keyword */
body .o { color: #666666 }                      /* Operator */
body .cm { color: #408080; font-style: italic } /* Comment.Multiline */
body .cp { color: #BC7A00 }                     /* Comment.Preproc */
body .c1 { color: #408080; font-style: italic } /* Comment.Single */
body .cs { color: #408080; font-style: italic } /* Comment.Special */
body .gd { color: #A00000 }                     /* Generic.Deleted */
body .ge { font-style: italic }                 /* Generic.Emph */
body .gr { color: #FF0000 }                     /* Generic.Error */
body .gh { color: #000080; font-weight: bold }  /* Generic.Heading */
body .gi { color: #00A000 }                     /* Generic.Inserted */
body .go { color: #808080 }                     /* Generic.Output */
body .gp { color: #000080; font-weight: bold }  /* Generic.Prompt */
body .gs { font-weight: bold }                  /* Generic.Strong */
body .gu { color: #800080; font-weight: bold }  /* Generic.Subheading */
body .gt { color: #0040D0 }                     /* Generic.Traceback */
body .kc { color: #954121 }                     /* Keyword.Constant */
body .kd { color: #954121; font-weight: bold }  /* Keyword.Declaration */
body .kn { color: #954121; font-weight: bold }  /* Keyword.Namespace */
body .kp { color: #954121 }                     /* Keyword.Pseudo */
body .kr { color: #954121; font-weight: bold }  /* Keyword.Reserved */
body .kt { color: #B00040 }                     /* Keyword.Type */
body .m { color: #666666 }                      /* Literal.Number */
body .s { color: #219161 }                      /* Literal.String */
body .na { color: #7D9029 }                     /* Name.Attribute */
body .nb { color: #954121 }                     /* Name.Builtin */
body .nc { color: #0000FF; font-weight: bold }  /* Name.Class */
body .no { color: #880000 }                     /* Name.Constant */
body .nd { color: #AA22FF }                     /* Name.Decorator */
body .ni { color: #999999; font-weight: bold }  /* Name.Entity */
body .ne { color: #D2413A; font-weight: bold }  /* Name.Exception */
body .nf { color: #0000FF }                     /* Name.Function */
body .nl { color: #A0A000 }                     /* Name.Label */
body .nn { color: #0000FF; font-weight: bold }  /* Name.Namespace */
body .nt { color: #954121; font-weight: bold }  /* Name.Tag */
body .nv { color: #19469D }                     /* Name.Variable */
body .ow { color: #AA22FF; font-weight: bold }  /* Operator.Word */
body .w { color: #bbbbbb }                      /* Text.Whitespace */
body .mf { color: #666666 }                     /* Literal.Number.Float */
body .mh { color: #666666 }                     /* Literal.Number.Hex */
body .mi { color: #666666 }                     /* Literal.Number.Integer */
body .mo { color: #666666 }                     /* Literal.Number.Oct */
body .sb { color: #219161 }                     /* Literal.String.Backtick */
body .sc { color: #219161 }                     /* Literal.String.Char */
body .sd { color: #219161; font-style: italic } /* Literal.String.Doc */
body .s2 { color: #219161 }                     /* Literal.String.Double */
body .se { color: #BB6622; font-weight: bold }  /* Literal.String.Escape */
body .sh { color: #219161 }                     /* Literal.String.Heredoc */
body .si { color: #BB6688; font-weight: bold }  /* Literal.String.Interpol */
body .sx { color: #954121 }                     /* Literal.String.Other */
body .sr { color: #BB6688 }                     /* Literal.String.Regex */
body .s1 { color: #219161 }                     /* Literal.String.Single */
body .ss { color: #19469D }                     /* Literal.String.Symbol */
body .bp { color: #954121 }                     /* Name.Builtin.Pseudo */
body .vc { color: #19469D }                     /* Name.Variable.Class */
body .vg { color: #19469D }                     /* Name.Variable.Global */
body .vi { color: #19469D }                     /* Name.Variable.Instance */
body .il { color: #666666 }                     /* Literal.Number.Integer.Long */
"""

def format_type(t):
    """
    Returns the type as a string.  If the type is an array, then it is prepended with []
    
    :Parameters:
      t
        The type as a dict. Keys: 'type', 'is_array'
    """
    s = ""
    if t.has_key('is_array') and t['is_array']:
        s = "[]%s" % t['type']
    else:
        s = t['type']
    if t.has_key('optional') and t['optional'] == True:
        s += " [optional]"
    return s

def docco_html(title, idl_parsed):
    """
    Translates a parsed Barrister IDL into HTML.  Returns a string containing the HTML.

    :Parameters:
      title
        Title of the document. Will be includined in <title> and <h1> tags
      idl_parsed
        Parsed representation of IDL to convert to HTML
    """
    sections = to_sections(idl_parsed)
    s = """<!DOCTYPE html>
<html>
<head>
  <title>%s</title>
  <meta http-equiv="content-type" content="text/html; charset=UTF-8">
  <style type="text/css">
  %s
  </style>
</head>
<body>
  <div id="container">
    <div id="background"></div>
    <table cellpadding="0" cellspacing="0">
      <thead>
        <tr>
          <th class="docs">
            <h1>
               %s
            </h1>
          </th>
          <th class="code">
          </th>
        </tr>
      </thead>
      <tbody>""" % (title, DOCCO_CSS, title)
    
    i = 0
    for sec in sections:
        i += 1
        s += """<tr id="section-%d">
            <td class="docs">
              <div class="pilwrap">
                <a class="pilcrow" href="#section-%d">&#182;</a>
              </div>
              %s
            </td>
            <td class="code">
              %s
            </td>
          </tr>""" % (i, i, sec['docs'], sec['code'])

    s += """</tbody>
    </table>
  </div>
</body>
</html>"""
    return s
    
def parse_enum(enum):
    """
    Returns a docco section for the given enum.

    :Parameters:
      enum
        Parsed IDL enum dict. Keys: 'comment', 'name', 'values'
    """
    docs = enum['comment']
    code = '<span class="k">enum</span> <span class="gs">%s</span> {\n' % enum['name']
    for v in enum["values"]:
        if v['comment']:
            for line in v['comment'].split("\n"):
                code += '    <span class="c1">// %s</span>\n' % line
        code += '    <span class="nv">%s</span>\n' % v['value']
    code += "}"

    return to_section(docs, code)

def parse_struct(s):
    """
    Returns a docco section for the given struct.

    :Parameters:
      s
        Parsed IDL struct dict. Keys: 'comment', 'name', 'extends', 'fields'
    """
    docs = s['comment']
    code = '<span class="k">struct</span> <span class="gs">%s</span>' % s['name']
    if s['extends']:
        code += ' extends <span class="gs">%s</span>' % s['extends']
    code += ' {\n'
    maxlen = 0
    for v in s["fields"]:
        if len(v['name']) > maxlen:
            maxlen = len(v['name'])

    maxlen += 1
    pad = ""
    for i in range(maxlen): pad += " "

    for v in s["fields"]:
        if v.has_key('comment') and v['comment']:
            for line in v['comment'].split("\n"):
                code += '    <span class="c1">// %s</span>\n' % line
        x = maxlen - len(v['name'])
        code += '    <span class="nv">%s</span>%s<span class="kt">%s</span>\n' % (v['name'], pad[:x], format_type(v))
    code += "}"

    return to_section(docs, code)

def parse_interface(iface):
    """
    Returns a docco section for the given interface.

    :Parameters:
      iface
        Parsed IDL interface dict. Keys: 'comment', 'name', 'returns', 'params'
    """
    sections = [ ]
    docs = iface['comment']
    code = '<span class="k">interface</span> <span class="gs">%s</span> {\n' % iface['name']
    for v in iface["functions"]:
        func_code = '    <span class="nf">%s</span>(' % v['name']
        i = 0
        for p in v["params"]:
            if i == 0: i = 1
            else: func_code += ", "
            func_code += '<span class="na">%s</span> <span class="kt">%s</span>' % (p['name'], format_type(p))
        func_code += ') <span class="kt">%s</span>\n' % format_type(v['returns'])
        if v.has_key('comment') and v['comment']:
            if code:
                sections.append(to_section(docs, code))
            docs = v['comment']
            code = func_code
        else:
            code += func_code
    code += "}"

    sections.append(to_section(docs, code))
    return sections

def wrap_code(code):
    """
    Wraps HTML tags around the given source code block

    :Parameters:
      code
        Source code to wrap
    """
    if code:
        return """<div class="highlight"><pre>%s</pre></div>""" % code
    else:
        return ""

def to_section(docs, code):
    """
    Returns a dict with keys: 'docs', 'code'.  docs are converted to markdown. code is converted
    to HTML using wrap_code.

    :Parameters:
      docs
        Documentation for the section in Markdown format
      code
        Source code for the section in plain text
    """
    return { "docs": markdown.markdown(docs), "code": wrap_code(code) }

def to_sections(idl_parsed):
    """
    Iterates through elements in idl_parsed list and returns a list of section dicts.
    Currently elements of type "comment", "enum", "struct", and "interface" are processed.

    :Parameters:
      idl_parsed
        Barrister parsed IDL
    """
    sections = []
    for entity in idl_parsed:
        if entity["type"] == "comment":
            sections.append(to_section(entity["value"], ""))
        elif entity["type"] == "enum":
            sections.append(parse_enum(entity))
        elif entity["type"] == "struct":
            sections.append(parse_struct(entity))
        elif entity["type"] == "interface":
            sections.extend(parse_interface(entity))
    return sections

if __name__ == "__main__":
    parser = optparse.OptionParser("usage: %prog [options] [json filename]")
    parser.add_option("-i", "--stdin", dest="stdin", action="store_true",
                      default=False, help="Read JSON from STDIN")
    parser.add_option("-t", "--title", dest="title",
                      default="", type="string",
                      help="page title")
    (options, args) = parser.parse_args()

    if options.stdin:
        j = sys.stdin.read()
    elif len(args) < 1:
        parser.error("Incorrect number of args")
    else:
        f = open(args[0])
        j = f.read()
        f.close()

    idl_parsed = json.loads(j)
    print docco_html(options.title, idl_parsed)



"""
    Parser lib for converting IDL prose into a parsed representation suitable for saving as JSON

    http://barrister.bitmechanic.com/

    :copyright: 2012 by James Cooper.
    :license: MIT, see LICENSE for more details.
"""

import time
import copy
import operator
import cStringIO
from plex import Scanner, Lexicon, Str, State, IGNORE, Begin, Any, AnyBut, AnyChar, Range, Rep
try:
    import json
except:
    import simplejson as json

def md5(s):
    try:
        import hashlib
        return hashlib.md5(s).hexdigest()
    except:
        import md5
        return md5.new(s).hexdigest()

native_types = [ "int", "float", "string", "bool" ]
letter       = Range("AZaz")
digit        = Range("09")
under        = Str("_")
ident        = letter + Rep(letter | digit | under)
arr_ident    = Str("[]") + ident
space        = Any(" \t\n\r")
comment      = Str("// ") | Str("//")
type_opts    = Str("[") + Rep(AnyBut("{}]\n")) + Str("]")

def parse(idl_text, name=None, validate=True, add_meta=True):
    if isinstance(idl_text, (str, unicode)):
        idl_text = cStringIO.StringIO(idl_text)

    scanner = IdlScanner(idl_text, name)
    scanner.parse()
    if validate:
        scanner.validate()
    if len(scanner.errors) == 0:
        if add_meta:
            scanner.add_meta()
        return scanner.parsed
    else:
        raise IdlParseException(scanner.errors)

class IdlParseException(Exception):

    def __init__(self, errors):
        Exception.__init__(self)
        self.errors = errors

    def __str__(self):
        s = ""
        for e in self.errors:
            if s != "":
                s += ", "
            s += "line: %d message: %s" % (e["line"], e["message"])
        return s

class IdlScanner(Scanner):

    def eof(self):
        if self.cur:
            self.add_error("Unexpected end of file")

    def add_meta(self):
        import barrister
        meta = {
            "type"              : "meta",
            "barrister_version" : barrister.__version__,
            "date_generated"    : int(time.time() * 1000),
            "checksum"          : self.get_checksum()
        }
        self.parsed.append(meta)

    def get_checksum(self):
        """
        Returns a checksum based on the IDL that ignores comments and ordering,
        but detects changes to types, parameter order, and enum values.
        """
        arr = [ ]
        for elem in self.parsed:
            if elem["type"] == "struct":
                s = ""
                fields = copy.copy(elem["fields"])
                fields.sort(key=operator.itemgetter("name"))
                for f in fields:
                    s += "\t%s\t%s\t%s\t%s" % (f["name"], f["type"], f["is_array"], f["optional"])
                arr.append("struct\t%s\t%s\t%s\n" % (elem["name"], elem["extends"], s))
            elif elem["type"] == "enum":
                s = "enum\t%s" % elem["name"]
                vals = copy.copy(elem["values"])
                vals.sort(key=operator.itemgetter("value"))
                for v in vals: s += "\t%s" % v["value"]
                s += "\n"
                arr.append(s)
            elif elem["type"] == "interface":
                s = "interface\t%s" % elem["name"]
                funcs = copy.copy(elem["functions"])
                funcs.sort(key=operator.itemgetter("name"))
                for f in funcs:
                    s += "[%s" % f["name"]
                    for p in f["params"]:
                        s += "\t%s\t%s" % (p["type"], p["is_array"])
                    ret = f["returns"]
                    s += "(%s\t%s\t%s)]" % (ret["type"], ret["is_array"], ret["optional"])
                s += "\n"
                arr.append(s)
        arr.sort()
        #print arr
        return md5(json.dumps(arr))

    def add_error(self, message, line=-1):
        if line < 0:
            (name, line, col) = self.position()
        self.errors.append({"line": line, "message": message})

    def begin_struct(self, text):
        self.check_dupe_name(text)
        self.cur = { "name" : text, "type" : "struct", "extends" : "",
                     "comment" : self.get_comment(), "fields" : [] }
        self.begin('start-block')

    def begin_enum(self, text):
        self.check_dupe_name(text)
        self.cur = { "name" : text, "type" : "enum", 
                     "comment" : self.get_comment(), "values" : [] }
        self.begin('start-block')

    def begin_interface(self, text):
        self.check_dupe_name(text)
        self.cur = { "name" : text, "type" : "interface", 
                     "comment" : self.get_comment(), "functions" : [] }
        self.begin('start-block')

    def check_dupe_name(self, name):
        if self.types.has_key(name):
            self.add_error("type %s already defined" % name)

    def check_not_empty(self, cur, list_name, printable_name):
        if len(cur[list_name]) == 0:
            self.add_error("%s must have at least one %s" % (cur["name"], printable_name))
            return False
        return True

    def start_block(self, text):
        t = self.cur["type"]
        if t == "struct":
            self.begin("fields")
        elif t == "enum":
            self.begin("values")
        elif t == "interface":
            self.begin("functions")
        else:
            raise Exception("Invalid type: %s" % t)

    def end_block(self, text):
        ok = False
        t = self.cur["type"]
        if t == "struct":
            ok = self.check_not_empty(self.cur, "fields", "field")
        elif t == "enum":
            ok = self.check_not_empty(self.cur, "values", "value")
        elif t == "interface":
            ok = self.check_not_empty(self.cur, "functions", "function")
        
        if ok:
            self.parsed.append(self.cur)
            self.types[self.cur["name"]] = self.cur

        self.cur = None
        self.begin('')

    def begin_field(self, text):
        self.field = { "name" : text }
        self.begin("field")

    def end_field(self, text):
        is_array = False
        if text.find("[]") == 0:
            text = text[2:]
            is_array = True
        self.field["type"] = text
        self.field["is_array"] = is_array
        self.field["comment"] = self.get_comment()
        self.field["optional"] = False
        self.type = self.field
        self.cur["fields"].append(self.field)
        self.field = None
        self.next_state = "fields"
        self.begin("type-opts")

    def begin_function(self, text):
        self.function = { "name" : text, "comment" : self.get_comment(), "params" : [ ] }
        self.begin("function-start")

    def begin_param(self, text):
        self.param = { "name" : text }
        self.begin("param")

    def end_param(self, text):
        is_array = False
        if text.find("[]") == 0:
            text = text[2:]
            is_array = True
        self.param["type"] = text
        self.param["is_array"] = is_array
        self.function["params"].append(self.param)
        self.param = None
        self.begin("end-param")

    def end_return(self, text):
        is_array = False
        if text.find("[]") == 0:
            text = text[2:]
            is_array = True
        self.function["returns"] = { "type": text, "is_array": is_array, "optional": False }
        self.type = self.function["returns"]
        self.next_state = "functions"
        self.cur["functions"].append(self.function)
        self.function = None
        self.begin("type-opts")

    def end_type_opts(self, text):
        text = text.strip()
        if text.startswith("[") and text.endswith("]"):
            text = text[1:-1]
        if text != "":
            if text == "optional":
                self.type["optional"] = True
            else:
                raise Exception("Invalid type option: %s" % text)
        self.type = None
        self.begin(self.next_state)
        self.next_state = None

    def end_type_opts_and_block(self, text):
        self.end_type_opts(text)
        self.end_block(text)

    def end_value(self, text):
        if not text in self.cur["values"]:
            val = { "value" : text, "comment" : self.get_comment() }
            self.last_comment = ""
            self.cur["values"].append(val)

    def get_comment(self):
        comment = ""
        if self.comment and len(self.comment) > 0:
            comment = "".join(self.comment)
        self.comment = None
        return comment

    def start_comment(self, text):
        if self.comment:
            self.comment.append("\n")
        else:
            self.comment = []
        self.prev_state = self.state_name
        self.begin("comment")

    def append_comment(self, text):
        self.comment.append(text)

    def append_field_options(self, text):
        self.field_options.append(text)

    def end_comment(self, text):
        self.begin(self.prev_state)
        self.prev_state = None

    def end_extends(self, text):
        if self.cur and self.cur["type"] == "struct":
            self.cur["extends"] = text
        else:
            self.add_error("extends is only supported for struct types")

    def add_comment_block(self, text):
        comment = self.get_comment()
        if comment:
            self.parsed.append({"type" : "comment", "value" : comment})

    lex = Lexicon([
            (Str("\n"),  add_comment_block),
            (space,      IGNORE),
            (Str('struct '),   Begin('struct-start')),
            (Str('enum '),   Begin('enum-start')),
            (Str('interface '),   Begin('interface-start')),
            (comment,    start_comment),
            State('struct-start', [
                    (ident,    begin_struct),
                    (space,    IGNORE),
                    (AnyChar, "Missing identifier") ]),
            State('enum-start', [
                    (ident,    begin_enum),
                    (space,    IGNORE),
                    (AnyChar, "Missing identifier") ]),
            State('interface-start', [
                    (ident,    begin_interface),
                    (space,    IGNORE),
                    (AnyChar, "Missing identifier") ]),
            State('start-block', [
                    (space, IGNORE),
                    (Str("extends"), Begin('extends')),
                    (Str('{'), start_block) ]),
            State('extends', [
                    (space, IGNORE),
                    (ident, end_extends),
                    (Str('{'), start_block) ]),
            State('fields', [
                    (ident,    begin_field),
                    (space,    IGNORE),
                    (comment, start_comment),
                    (Str('{'), 'invalid'),
                    (Str('}'), end_block) ]),
            State('field', [
                    (ident,    end_field),
                    (arr_ident, end_field),
                    (Str("\n"), 'invalid'),
                    (space,    IGNORE),
                    (Str('{'), 'invalid'),
                    (Str('}'), 'invalid') ]),
            State('functions', [
                    (ident,    begin_function),
                    (space,    IGNORE),
                    (comment,  start_comment),
                    (Str('{'), 'invalid'),
                    (Str('}'), end_block) ]),
            State('function-start', [
                    (Str("("), Begin('params')),
                    (Str("\n"), 'invalid'),
                    (space,    IGNORE) ]),
            State('params', [
                    (ident,    begin_param),
                    (space,    IGNORE),
                    (Str(")"), Begin('function-return')) ]),
            State('end-param', [
                    (space, IGNORE),
                    (Str(","), Begin('params')),
                    (Str(")"), Begin('function-return')) ]),
            State('param', [
                    (ident,    end_param),
                    (arr_ident, end_param),
                    (space,    IGNORE) ]),
            State('function-return', [
                    (space,    IGNORE),
                    (ident,    end_return),
                    (arr_ident, end_return) ]),
            State('type-opts', [
                    (type_opts, end_type_opts),
                    (Str("\n"), end_type_opts),
                    (Str('}'),  end_block),
                    (space,    IGNORE),
                    (Str('{'),  'invalid') ]),
            State('end-function', [
                    (Str("\n"), Begin('functions')),
                    (space, IGNORE) ]),
            State('values', [
                    (ident,    end_value),
                    (space,    IGNORE),
                    (comment,  start_comment),
                    (Str('{'), 'invalid'),
                    (Str('}'), end_block) ]),
            State('comment', [
                    (Str("\n"),     end_comment),
                    (AnyChar, append_comment) ])
            ])

    def __init__(self, f, name):
        Scanner.__init__(self, self.lex, f, name)
        self.parsed = [ ]
        self.errors = [ ]
        self.types = { }
        self.comment = None
        self.cur = None

    def parse(self):
        while True:
            (t, name) = self.read()
            if t is None:
                break
            else:
                self.add_error(t)
                break

    def validate_type(self, cur_type, types, level):
        level += 1

        cur_type = self.strip_array_chars(cur_type)

        if cur_type in native_types or cur_type in types:
            pass
        elif not self.types.has_key(cur_type):
            self.add_error("undefined type: %s" % cur_type, line=0)
        else:
            cur = self.types[cur_type]
            types.append(cur_type)
            if cur["type"] == "struct":
                if cur["extends"] != "":
                    self.validate_type(cur["extends"], types, level)
                for f in cur["fields"]:
                    self.validate_type(f["type"], types, level)
            elif cur["type"] == "interface":
                # interface types must be top-level, so if len(types) > 1, we
                # know this interface was used as a type in a function or struct
                if level > 1:
                    msg = "interface %s cannot be a field type" % cur["name"]
                    self.add_error(msg, line=0)
                else:
                    for f in cur["functions"]:
                        types = [ ]
                        for p in f["params"]:
                            self.validate_type(p["type"], types, 1)
                        self.validate_type(f["returns"]["type"], types, 1)

    def add_parent_fields(self, s, names, types):
        if s["extends"] in native_types:
            self.add_error("%s cannot extend %s" % (s["name"], s["extends"]), line=0)
        elif self.types.has_key(s["extends"]):
            if s["name"] not in types:
                types.append(s["name"])
                parent = self.types[s["extends"]]
                if parent["type"] == "struct":
                    for f in parent["fields"]:
                        if f["name"] not in names:
                            names.append(f["name"])
                    self.add_parent_fields(parent, names, types)
                else:
                    self.add_error("%s cannot extend %s %s" % (s["name"], parent["type"], parent["name"]), line=0)

    def validate_struct_extends(self, s):
        names = []
        self.add_parent_fields(s, names, [])
        for f in s["fields"]:
            if f["name"] in names:
                self.add_error("%s cannot redefine parent field %s" % (s["name"], f["name"]), line=0)

    def contains_cycle(self, name, types):
        name = self.strip_array_chars(name)
        if self.types.has_key(name):
            t = self.types[name]
            if t["type"] == "struct":
                if name in types:
                    self.add_error("cycle detected in: %s %s" % (t["type"], name), line=0)
                    return True
                else:
                    types.append(name)
                    if self.contains_cycle(t["extends"], types):
                        return True
                    for f in t["fields"]:
                        # use a copy of the type list to keep function checks separate
                        if self.contains_cycle(f["type"], types[:]):
                            return True
        return False

    def strip_array_chars(self, name):
        if name.find("[]") == 0:
            return name[2:]
        return name

    def validate(self):
        for t in self.parsed:
            if t["type"] == "comment":
                pass
            elif not self.contains_cycle(t["name"], []):
                self.validate_type(t["name"], [], 0)
                if t["type"] == "struct":
                    self.validate_struct_extends(t)



#!/usr/bin/env python

"""
    barrister
    ~~~~~~~~~

    A RPC toolkit for building lightweight reliable services.  Ideal for
    both static and dynamic languages.

    :copyright: (c) 2012 by James Cooper.
    :license: MIT, see LICENSE for more details.
"""

import uuid
import time
import unittest
import barrister
from barrister.parser import parse

idl = """
struct User {
    userId string
    password string
    email string
    emailVerified bool
    dateCreated int
    age float [optional]
}

enum Status {
    ok
    invalid
    error
}

struct Response {
    status Status
    message string
}

struct CountResponse extends Response {
    count int
}

struct CreateUserResponse extends Response {
    userId string
}

struct UserResponse extends Response {
    user User
}

struct UsersResponse extends Response {
    users []User
}

interface UserService {
    get(userId string) UserResponse
    create(user User) CreateUserResponse
    update(user User) Response
    validateEmail(userId string) Response
    changePassword(userId string, oldPass string, newPass string) Response
    countUsers() CountResponse
    getAll(userIds []string) UsersResponse
}
"""

def newUser(userId="abc123", email=None):
    return { "userId" : userId, "password" : u"pw", "email" : email,
      "emailVerified" : False, "dateCreated" : 1, "age" : 3.3 }

def now_millis():
    return int(time.time() * 1000)

class UserServiceImpl(object):

    def __init__(self):
        self.users = { }

    def get(self, userId):
        resp = self._resp("ok", "user created")
        resp["user"] = self.users[userId]
        return resp

    def create(self, user):
        resp = self._resp("ok", "user created")
        userId = uuid.uuid4().hex
        user["dateCreated"] = now_millis()
        resp["userId"] = userId
        self.users[userId] = user
        return resp

    def update(self, user):
        userId = user["userId"]
        self.users[userId] = user
        return self._resp("ok", "user updated")

    def validateEmail(self, userId):
        return self._resp("ok", "user updated")

    def changePassword(self, userId, oldPass, newPass):
        return self._resp("ok", "password updated")

    def countUsers(self):
        resp = self._resp("ok", "ok")
        resp["count"] = len(self.users)
        return resp

    def getAll(self, userIds):
        return { "status": "ok", "message" : "users here", "users": [] }

    def _resp(self, status, message):
        return { "status" : status, "message" : message }

class RuntimeTest(unittest.TestCase):

    def setUp(self):
        contract = barrister.Contract(parse(idl))
        self.user_svc = UserServiceImpl()
        self.server = barrister.Server(contract)
        self.server.add_handler("UserService", self.user_svc)

        transport = barrister.InProcTransport(self.server)
        self.client = barrister.Client(transport)

    def test_add_handler_invalid(self):
        self.assertRaises(barrister.RpcException, self.server.add_handler, "foo", self.user_svc)

    def test_user_crud(self):
        svc = self.client.UserService
        user = newUser(email="foo@example.com")
        del user["age"]  # field is optional
        resp = svc.create(user)
        self.assertTrue(resp["userId"])
        user2 = svc.get(resp["userId"])["user"]
        self.assertEquals(user["email"], user2["email"])
        self.assertTrue(user["dateCreated"] > 0)
        self.assertEquals("ok", svc.changePassword("123", "oldpw", "newpw")["status"])
        self.assertEquals(1, svc.countUsers()["count"])
        svc.getAll([])

    def test_invalid_req(self):
        svc = self.client.UserService
        cases = [ 
            [ svc.get ],  # too few args
            [ svc.get, 1, 2 ], # too many args
            [ svc.get, 1 ], # wrong type
            [ svc.create, None ], # wrong type
            [ svc.create, 1 ], # wrong type
            [ svc.create, { "UserId" : "1" } ], # unknown param
            [ svc.create, { "userId" : 1 } ], # wrong type
            [ svc.getAll, { } ], # wrong type
            [ svc.getAll, [ 1 ] ] # wrong type
            ]
        for c in cases:
            try:
                if len(c) > 1:
                    c[0](c[1])
                else:
                    c[0]()
                self.fail("Expected RpcException for: %s" % str(c))
            except barrister.RpcException:
                pass

    def test_invalid_resp(self):
        svc = self.client.UserService
        responses = [ 
            { }, # missing fields
            { "status" : "blah" }, # invalid enum
            { "status" : "ok", "message" : 1 }, # invalid type
            { "status" : "ok", "message" : "good", "blarg" : True }, # invalid field
            { "status" : "ok", "message" : "good", "user" : { # missing password field
                    "userId" : "123", "email" : "foo@bar.com",
                    "emailVerified" : False, "dateCreated" : 1, "age" : 3.3 } },
            { "status" : "ok", "message" : "good", "user" : { # missing password field
                    "userId" : "123", "email" : "foo@bar.com",
                    "emailVerified" : False, "dateCreated" : 1, "age" : 3.3 } },
            { "status" : "ok", "user" : { # missing message field
                    "userId" : "123", "email" : "foo@bar.com",
                    "emailVerified" : False, "dateCreated" : 1, "age" : 3.3 } }
            ]
        for resp in responses:
            self.user_svc.get = lambda id: resp
            try:
                svc.get("123")
                self.fail("Expected RpcException for response: %s" % str(resp))
            except barrister.RpcException:
                pass

    def test_batch(self):
        batch = self.client.start_batch()
        batch.UserService.create(newUser(userId="1", email="foo@bar.com"))
        batch.UserService.create(newUser(userId="2", email="foo@bar.com"))
        batch.UserService.countUsers()
        results = batch.send()
        self.assertEquals(3, len(results))
        self.assertEquals(results[0].result["message"], "user created")
        self.assertEquals(results[1].result["message"], "user created")
        self.assertEquals(2, results[2].result["count"])

    def _test_bench(self):
        start = time.time()
        stop = start+1
        num = 0
        while time.time() < stop:
            self.client.UserService.countUsers()
            num += 1
        elapsed = time.time() - start
        print "test_bench: num=%d microsec/op=%d" % (num, (elapsed*1000000)/num)

        start = time.time()
        stop = start+1
        num = 0
        while time.time() < stop:
            self.client.UserService.create(newUser())
            num += 1
        elapsed = time.time() - start
        print "test_bench: num=%d microsec/op=%d" % (num, (elapsed*1000000)/num)

if __name__ == "__main__":
    unittest.main()




"""
Barrister
---------

Polyglot RPC system that can enforce types expressed in an easy to write IDL.
Designed to be enjoyable to use from both static and dynamic languages.

Links
`````

* `main site <http://barrister.bitmechanic.com/>`_
* `GitHub repo <https://github.com/coopernurse/barrister>`_

"""
from distutils.core import setup

setup(
    name='barrister',
    version='0.1.2',
    url='https://github.com/coopernurse/barrister',
    scripts=['bin/barrister'],
    packages=['barrister',],
    license='MIT',
    author='James Cooper',
    author_email='james@bitmechanic.com',
    description='Polyglot RPC',
    long_description=__doc__,
    install_requires=[
        'Markdown',
        'plex'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: System :: Distributed Computing',
        'Topic :: Software Development :: Libraries'
    ]
)



"""
    Barrister runtime for Python.  Includes all classes used when writing a client or server.

    :copyright: 2012 by James Cooper.
    :license: MIT, see LICENSE for more details.
"""
import urllib2
import uuid
import itertools
import logging
try:
    import json
except: 
    import simplejson as json

# JSON-RPC standard error codes
ERR_PARSE = -32700
ERR_INVALID_REQ = -32600
ERR_METHOD_NOT_FOUND = -32601
ERR_INVALID_PARAMS = -32602
ERR_INTERNAL = -32603

# Our extensions
ERR_UNKNOWN = -32000
ERR_INVALID_RESP = -32001

def contract_from_file(fname):
    """
    Loads a Barrister IDL JSON from the given file and returns a Contract class

    :Parameters:
      fname
        Filename containing Barrister IDL JSON to load
    """
    f = open(fname)
    j = f.read()
    f.close()
    return Contract(json.loads(j))

def unpack_method(method):
    """
    Given a JSON-RPC method in:  [interface].[function] notation, returns a tuple of the interface
    name and function.

    For example, unpack_method("MyService.LoadUser") would return: ("MyService", "LoadUser")

    :Parameters:
      method
        String method name
    """
    pos = method.find(".")
    if pos == -1:
        raise RpcException(ERR_METHOD_NOT_FOUND, "Method not found: %s" % method)

    iface_name = method[:pos]
    func_name  = method[pos+1:]
    return iface_name, func_name

def idgen_uuid():
    """
    Generates a uuid4 (random) and returns the hex representation as a string
    """
    return uuid.uuid4().hex

idgen_seq_counter = itertools.count()
def idgen_seq():
    """
    Generates an ID using itertools.count() and returns it as a string
    """
    return str(idgen_seq_counter.next())

def err_response(reqid, code, msg, data=None):
    """
    Formats a JSON-RPC error as a dict with keys: 'jsonrpc', 'id', 'error'
    """
    err = { "code": code, "message": msg }
    if data:
        err["data"] = data
    return { "jsonrpc": "2.0", "id": reqid, "error": err }

def safe_get(d, key, def_val=None):
    """
    Helper function to fetch value from a dictionary
    
    * `d` - Dictionary to fetch value from
    * `key` - Key to lookup in dictionary
    * `def_val` - Default value to return if dict does not have a member with key
    """
    if d.has_key(key):
        return d[key]
    else:
        return def_val

class RpcException(Exception, json.JSONEncoder):
    """
    Represents a JSON-RPC style exception.  Server implementations should raise this
    exception if they wish to communicate error codes back to Barrister clients.
    """

    def __init__(self, code, msg="", data=None):
        """
        Creates a new RpcException

        :Parameters:
          code
            Integer representing the error type. Applications may use any positive integer.
          msg
            Human readable description of the error
          data
            Optional extra info about the error. Should be a string, int, or list or dict of strings/ints
        """
        self.code = code
        self.msg  = msg
        self.data = data

    def __str__(self):
        s = "RpcException: code=%d msg=%s" % (self.code, self.msg)
        if self.data:
            s += "%s data=%s" % (s, str(self.data))
        return s

class RequestContext(object):
    """
    Stores state about a single request, including properties passed
    into Server.call
    """

    def __init__(self, props, req):
        """
        Creates a new RequestContext

        :Parameters:
          props
            Dict of meta properties for this request
          req
            Dict that represents a single JSON-RPC request
        """
        self.props    = props
        self.request  = req
        self.response = None
        self.error    = None

    def func_name(self):
        return unpack_method(self.request["method"])[1]

    def get_prop(self, key, default_val=None):
        """
        Returns a property set on the context.

        :Parameters:
          key
            String key to lookup in the context props dict
          default_val
            Value to return if key is not set on the context props
        """
        if self.props.has_key(key):
            return self.props[key]
        else:
            return default_val

    def set_error(self, code, msg, data=None):
        """
        Set an error on this request, which will prevent request execution.
        Should only be called from "pre" hook methods.  If called from a post hook, this
        operation will be ignored.

        :Parameters:
          code
            Integer error code
          msg
            String description of the error
          data
            Optional additional info about the error. Should be a primitive, or a list or
            dict of primitives to avoid serialization issues.
        """
        self.error = err_response(self.request["id"], code, msg, data)

class Filter(object):
    """
    Base filter class that implements pre and post functions, but no-ops for both.
    Subclass this and override pre/post to add filter functionality for your app.
    """
    
    def pre(self, context):
        """
        Pre-Handler hook.  Called before the RPC request handler is invoked.
        If context.send_error is called by any pre filter, then the request handler will
        not be invoked, and the error will be returned instead.

        :Parameters:
          context
            RequestContext instance for this request
        """
        pass

    def post(self, context):
        """
        Post-Handler hook.  Called after the RPC request handler is invoked.
        Post handlers can inspect and log the response, but should not alter it.

        :Parameters:
          context
            RequestContext instance for this request
        """
        pass

class Server(object):
    """
    Dispatches requests to user created handler classes based on method name.
    Also responsible for validating requests and responses to ensure they conform to the
    IDL Contract.
    """

    def __init__(self, contract, validate_request=True, validate_response=True):
        """
        Creates a new Server

        :Parameters:
          contract
            Contract instance that this server should use
          validate_request
            If True, requests will be validated against the Contract and rejected if they are malformed
          validate_response
            If True, responses from handler methods will be validated against the Contract and rejected
            if they are malformed
        """
        logging.basicConfig()
        self.log = logging.getLogger("barrister")
        self.validate_req  = validate_request
        self.validate_resp = validate_response
        self.contract = contract
        self.handlers = { }
        self.filters = None

    def add_handler(self, iface_name, handler):
        """
        Associates the given handler with the interface name.  If the interface does not exist in
        the Contract, an RpcException is raised.

        :Parameters:
          iface_name
            Name of interface that this handler implements
          handler
            Instance of a class that implements all functions defined on the interface
        """
        if self.contract.has_interface(iface_name):
            self.handlers[iface_name] = handler
        else:
            raise RpcException(ERR_INVALID_REQ, "Unknown interface: '%s'", iface_name)

    def set_filters(self, filters):
        """
        Sets the filters for the server.

        :Parameters:
          filters
            List of filters to set on this server, or None to remove all filters.
            Elements in list should subclass Filter
        """
        if filters == None or isinstance(filters, (tuple, list)):
            self.filters = filters
        else:
            self.filters = [ filters ]

    def call_json(self, req_json, props=None):
        """
        Deserializes req_json as JSON, invokes self.call(), and serializes result to JSON.
        Returns JSON encoded string.

        :Parameters:
          req_json
            JSON-RPC request serialized as JSON string
          props
            Application defined properties to set on RequestContext for use with filters. 
            For example: authentication headers.  Must be a dict.
        """
        try:
            req = json.loads(req_json)
        except:
            msg = "Unable to parse JSON: %s" % req_json
            return json.dumps(err_response(None, -32700, msg))
        return json.dumps(self.call(req, props))

    def call(self, req, props=None):
        """
        Executes a Barrister request and returns a response.  If the request is a list, then the
        response will also be a list.  If the request is an empty list, a RpcException is raised.

        :Parameters:
          req
            The request. Either a list of dicts, or a single dict.
          props
            Application defined properties to set on RequestContext for use with filters. 
            For example: authentication headers.  Must be a dict.
        """
        resp = None

        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Request: %s" % str(req))

        if isinstance(req, list):
            if len(req) < 1:
                resp = err_response(None, ERR_INVALID_REQ, "Invalid Request. Empty batch.")
            else:
                resp = [ ]
                for r in req:
                    resp.append(self._call_and_format(r, props))
        else:
            resp = self._call_and_format(req, props)

        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Response: %s" % str(resp))
        return resp
    
    def _call_and_format(self, req, props=None):
        """
        Invokes a single request against a handler using _call() and traps any errors,
        formatting them using _err().  If the request is successful it is wrapped in a 
        JSON-RPC 2.0 compliant dict with keys: 'jsonrpc', 'id', 'result'.

        :Parameters:
          req
            A single dict representing a single JSON-RPC request
          props
            Application defined properties to set on RequestContext for use with filters. 
            For example: authentication headers.  Must be a dict.
        """
        if not isinstance(req, dict):
            return err_response(None, ERR_INVALID_REQ, 
                             "Invalid Request. %s is not an object." % str(req))

        reqid = None
        if req.has_key("id"):
            reqid = req["id"]

        if props == None:
            props = { }
        context = RequestContext(props, req)

        if self.filters:
            for f in self.filters:
                f.pre(context)

        if context.error:
            return context.error

        resp = None
        try:
            result = self._call(context)
            resp = { "jsonrpc": "2.0", "id": reqid, "result": result }
        except RpcException, e:
            resp = err_response(reqid, e.code, e.msg, e.data)
        except:
            self.log.exception("Error processing request: %s" % str(req))
            resp = err_response(reqid, ERR_UNKNOWN, "Server error. Check logs for details.")
        
        if self.filters:
            context.response = resp
            for f in self.filters:
                f.post(context)

        return resp

    def _call(self, context):
        """
        Executes a single request against a handler.  If the req.method == 'barrister-idl', the
        Contract IDL JSON structure is returned.  Otherwise the method is resolved to a handler
        based on the interface name, and the appropriate function is called on the handler.

        :Parameter:
          req
            A dict representing a valid JSON-RPC 2.0 request.  'method' must be provided.
        """
        req = context.request
        if not req.has_key("method"):
            raise RpcException(ERR_INVALID_REQ, "Invalid Request. No 'method'.")

        method = req["method"]

        if method == "barrister-idl":
            return self.contract.idl_parsed

        iface_name, func_name = unpack_method(method)

        if self.handlers.has_key(iface_name):
            iface_impl = self.handlers[iface_name]
            func = getattr(iface_impl, func_name)
            if func:
                if req.has_key("params"):
                    params = req["params"]
                else:
                    params = [ ]

                if self.validate_req:
                    self.contract.validate_request(iface_name, func_name, params)

                if hasattr(iface_impl, "barrister_pre"):
                    pre_hook = getattr(iface_impl, "barrister_pre")
                    pre_hook(context, params)

                if params:
                    result = func(*params)
                else:
                    result = func()

                if self.validate_resp:
                    self.contract.validate_response(iface_name, func_name, result)
                return result
            else:
                msg = "Method '%s' not found" % (method)
                raise RpcException(ERR_METHOD_NOT_FOUND, msg)
        else:
            msg = "No implementation of '%s' found" % (iface_name)
            raise RpcException(ERR_METHOD_NOT_FOUND, msg)        

class HttpTransport(object):
    """
    A client transport that uses urllib2 to make requests against a HTTP server.
    """

    def __init__(self, url, handlers=None, headers=None):
        """
        Creates a new HttpTransport

        :Parameters:
          url
            URL of the server endpoint
          handlers
            Optional list of handlers to pass to urllib2.build_opener()
          headers
            Optional list of HTTP headers to set on requests.  Note that Content-Type will always be set
            automatically to "application/json"
        """
        if not headers:
            headers = { }
        headers['Content-Type'] = 'application/json'
        self.url = url
        self.headers = headers
        if handlers:
            self.opener = urllib2.build_opener(*handlers)
        else:
            self.opener = urllib2.build_opener()
        
    def request(self, req):
        """
        Makes a request against the server and returns the deserialized result.

        :Parameters:
          req
            List or dict representing a JSON-RPC formatted request
        """
        data = json.dumps(req)
        req = urllib2.Request(self.url, data, self.headers)
        f = self.opener.open(req)
        resp = f.read()
        f.close()
        return json.loads(resp)

class InProcTransport(object):
    """
    A client transport that invokes calls directly against a Server instance in process.
    This is useful for quickly unit testing services without having to go over the network.
    """
    def __init__(self, server):
        """ 
        Creates a new InProcTransport for the given Server

        :Parameters:
          server
            Barrister Server instance to bind this transport to
        """
        self.server = server

    def request(self, req):
        """
        Performs request against the given server.

        :Parameters:
          req
            List or dict representing a JSON-RPC formatted request
        """
        return self.server.call(req)

class Client(object):
    """
    Main class for consuming a server implementation.  Given a transport it loads the IDL from
    the server and creates proxy objects that can be called like local classes from your 
    application code.  

    With the exception of start_batch, you generally never need to use the methods provided by this
    class directly.

    For example:

    ::

      client = barrister.Client(barrister.HttpTransport("http://localhost:8080/OrderManagement"))
      status = client.OrderService.getOrderStatus("order-123")

    """

    def __init__(self, transport, validate_request=True, validate_response=True,
                 id_gen=idgen_uuid):
        """
        Creates a new Client for the given transport. When the constructor is called the
        client immediately makes a request to the server to load the IDL.  It then creates
        proxies for each interface in the IDL.  After constructing a client you can immediately
        begin making requests against the proxies.

        :Parameters:
          transport
            Transport object to use to make requests
          validate_request
            If True, the request will be validated against the Contract and a RpcException raised if 
            it does not match the IDL
          validate_response
            If True, the response will be validated against the Contract and a RpcException raised if 
            it does not match the IDL
          id_gen
            A callable to use to create request IDs.  JSON-RPC request IDs are only used by Barrister
            to correlate requests with responses when using a batch, but your application may use them
            for logging or other purposes.  UUIDs are used by default, but you can substitute another
            function if you prefer something shorter.
        """
        logging.basicConfig()
        self.log = logging.getLogger("barrister")
        self.transport = transport
        self.validate_req  = validate_request
        self.validate_resp = validate_response
        self.id_gen = id_gen
        req = {"jsonrpc": "2.0", "method": "barrister-idl", "id": "1"}
        resp = transport.request(req)
        self.contract = Contract(resp["result"])
        for k, v in self.contract.interfaces.items():
            setattr(self, k, InterfaceClientProxy(self, v))

    def get_meta(self):
        """
        Returns the dict of metadata from the Contract
        """
        return self.contract.meta

    def call(self, iface_name, func_name, params):
        """
        Makes a single RPC request and returns the result.

        :Parameters:
          iface_name
            Interface name to call
          func_name
            Function to call on the interface
          params
            List of parameters to pass to the function
        """
        req  = self.to_request(iface_name, func_name, params)
        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Request: %s" % str(req))
        resp = self.transport.request(req)
        if self.log.isEnabledFor(logging.DEBUG):
            self.log.debug("Response: %s" % str(resp))
        return self.to_result(iface_name, func_name, resp)

    def to_request(self, iface_name, func_name, params):
        """
        Converts the arguments to a JSON-RPC request dict.  The 'id' field is populated
        using the id_gen function passed to the Client constructor.

        If validate_request==True on the Client constructor, the params are validated
        against the expected types for the function and a RpcException raised if they are
        invalid.

        :Parameters:
          iface_name
            Interface name to call
          func_name
            Function to call on the interface
          params
            List of parameters to pass to the function
        """
        if self.validate_req:
            self.contract.validate_request(iface_name, func_name, params)
            
        method = "%s.%s" % (iface_name, func_name)
        reqid = self.id_gen()
        return { "jsonrpc": "2.0", "id": reqid, "method": method, "params": params }

    def to_result(self, iface_name, func_name, resp):
        """
        Takes a JSON-RPC response and checks for an "error" slot. If it exists,
        a RpcException is raised.  If no "error" slot exists, the "result" slot is 
        returned.

        If validate_response==True on the Client constructor, the result is validated
        against the expected return type for the function and a RpcException raised if it is
        invalid.

        :Parameters:
          iface_name
            Interface name that was called
          func_name
            Function that was called on the interface
          resp
            Dict formatted as a JSON-RPC response
        """
        if resp.has_key("error"):
            e = resp["error"]
            data = None
            if e.has_key("data"):
                data = e["data"]
            raise RpcException(e["code"], e["message"], data)
            
        result = resp["result"]
        
        if self.validate_resp:
            self.contract.validate_response(iface_name, func_name, result)
        return result

    def start_batch(self):
        """
        Returns a new Batch object for the Client that can be used to make multiple RPC calls
        in a single request.
        """
        return Batch(self)

class InterfaceClientProxy(object):
    """
    Internal class used by the Client.  One instance is created per Client per interface found
    on the IDL returned from the server.
    """

    def __init__(self, client, iface):
        """
        Creates a new InterfaceClientProxy

        :Parameters:
          client
            Client instance to associate with this proxy
          iface
            Dict interface from the parsed IDL.  All functions defined on this interface will
            be defined on this proxy class as callables.
        """
        self.client = client
        iface_name = iface.name
        for func_name, func in iface.functions.items():
            setattr(self, func_name, self._caller(iface_name, func_name))

    def _caller(self, iface_name, func_name):
        """
        Returns a function for the given interface and function name.  When invoked it
        calls client.call() with the correct arguments.
        
        :Parameters:
          iface_name
            Name of interface to call when invoked
          func_name
            Name of function to call when invoked
          params
            Params pass to function from the calling application
        """
        def caller(*params):
            return self.client.call(iface_name, func_name, params)
        return caller        

class Batch(object):
    """
    Provides a way to batch requests together in a single call.  This class functions
    similiarly to the Client class.  InterfaceClientProxy instances are attached to the Batch
    instance, but when the application code calls them, the params are stored in memory until
    `batch.send()` is called.
    """

    def __init__(self, client):
        """
        Creates a new Batch for the given Client instance.  Rarely called directly.  Use
        client.start_batch() instead.

        :Parameters:
          client
            Client instance to associate with this Batch
        """
        self.client = client
        self.req_list = [ ]
        self.sent = False
        for k, v in client.contract.interfaces.items():
            setattr(self, k, InterfaceClientProxy(self, v))

    def call(self, iface_name, func_name, params):
        """
        Implements the call() function with same signature as Client.call().  Raises
        a RpcException if send() has already been called on this batch.  Otherwise
        appends the request to an internal list.

        This method is not commonly called directly.
        """
        if self.sent:
            raise Exception("Batch already sent. Cannot add more calls.")
        else:
            req = self.client.to_request(iface_name, func_name, params)
            self.req_list.append(req)

    def send(self):
        """
        Sends the batch request to the server and returns a list of RpcResponse
        objects.  The list will be in the order that the requests were made to
        the batch.  Note that the RpcResponse objects may contain an error or a 
        successful result.  When you iterate through the list, you must test for
        response.error.
        
        send() may not be called more than once.
        """
        if self.sent:
            raise Exception("Batch already sent. Cannot send() again.")
        else:
            self.sent = True
            results = self.client.transport.request(self.req_list)
            
            id_to_method = { }
            by_id = { }
            for res in results:
                reqid = res["id"]
                by_id[reqid] = res

            in_req_order = [ ]
            for req in self.req_list:
                reqid  = req["id"]
                result = None
                error  = None
                resp   = safe_get(by_id, reqid)
                if resp == None:
                    msg = "Batch response missing result for request id: %s" % reqid
                    error = RpcException(ERR_INVALID_RESP, msg)
                else:
                    r_err = safe_get(resp, "error")
                    if r_err == None:
                        result = resp["result"]
                    else:
                        error = RpcException(r_err["code"], r_err["message"], safe_get(r_err, "data"))
                in_req_order.append(RpcResponse(req, result, error))
            return in_req_order
                

class RpcResponse(object):
    """
    Represents a single response in a batch call.  Has the following properties:
    
    * `request` - JSON-RPC request dict
    * `result`  - Result from this call. Set to None if there was an error.
    * `error`   - RpcException instance.  Set to None if call was successful.
    """
    
    def __init__(self, request, result, error):
        self.request = request
        self.result  = result
        self.error   = error

class Contract(object):
    """
    Represents a single IDL file
    """

    def __init__(self, idl_parsed):
        """
        Creates a new Contract from the parsed IDL JSON

        :Parameters:
          idl_parsed
            Barrister parsed IDL as a list of dicts
        """
        self.idl_parsed = idl_parsed
        self.interfaces = { }
        self.structs = { }
        self.enums = { }
        self.meta = { }
        for e in idl_parsed:
            if e["type"] == "struct":
                self.structs[e["name"]] = Struct(e, self)
            elif e["type"] == "enum":
                self.enums[e["name"]] = Enum(e)
            elif e["type"] == "interface":
                self.interfaces[e["name"]] = Interface(e, self)
            elif e["type"] == "meta":
                for k,v in e.items():
                    if k != "type":
                        self.meta[k] = v

    def validate_request(self, iface_name, func_name, params):
        """
        Validates that the given params match the expected length and types for this 
        interface and function.  

        Returns two element tuple: (bool, string)

        - `bool` - True if valid, False if not
        - `string` - Description of validation error, or None if valid

        :Parameters:
          iface_name
            Name of interface
          func_name
            Name of function
          params
            List of params to validate against this function
        """
        self.interface(iface_name).function(func_name).validate_params(params)

    def validate_response(self, iface_name, func_name, resp):
        """
        Validates that the response matches the return type for the function  

        Returns two element tuple: (bool, string)

        - `bool` - True if valid, False if not
        - `string` - Description of validation error, or None if valid

        :Parameters:
          iface_name
            Name of interface
          func_name
            Name of function
          resp
            Result from calling the function
        """
        self.interface(iface_name).function(func_name).validate_response(resp)

    def get(self, name):
        """
        Returns the struct, enum, or interface with the given name, or raises RpcException if
        no elements match that name.

        :Parameters:
          name
            Name of struct/enum/interface to return
        """
        if self.structs.has_key(name):
            return self.structs[name]
        elif self.enums.has_key(name):
            return self.enums[name]
        elif self.interfaces.has_key(name):
            return self.interfaces[name]
        else:
            raise RpcException(ERR_INVALID_PARAMS, "Unknown entity: '%s'" % name)

    def struct(self, struct_name):
        """
        Returns the struct with the given name, or raises RpcException if no struct matches
        """
        if self.structs.has_key(struct_name):
            return self.structs[struct_name]
        else:
            raise RpcException(ERR_INVALID_PARAMS, "Unknown struct: '%s'", struct_name)

    def has_interface(self, iface_name):
        """
        Returns True if an interface exists with the given name.  Otherwise returns False
        """
        return self.interfaces.has_key(iface_name)

    def interface(self, iface_name):
        """
        Returns the interface with the given name, or raises RpcException if no interface matches
        """
        if self.has_interface(iface_name):
            return self.interfaces[iface_name]
        else:
            raise RpcException(ERR_INVALID_PARAMS, "Unknown interface: '%s'", iface_name)

    def validate(self, expected_type, is_array, val):
        """
        Validates that the expected type matches the value

        Returns two element tuple: (bool, string)

        - `bool` - True if valid, False if not
        - `string` - Description of validation error, or None if valid

        :Parameters:
          expected_type
            string name of the type expected. This may be a Barrister primitive, or a user defined type.
          is_array
            If True then require that the val be a list
          val
            Value to validate against the expected type
        """
        if val == None:
            if expected_type.optional:
                return True, None
            else:
                return False, "Value cannot be null"
        elif is_array:
            if not isinstance(val, list):
                return self._type_err(val, "list")
            else:
                for v in val:
                    ok, msg = self.validate(expected_type, False, v)
                    if not ok:
                        return ok, msg
        elif expected_type.type == "int":
            if not isinstance(val, (long, int)):
                return self._type_err(val, "int")
        elif expected_type.type == "float":
            if not isinstance(val, (float, int, long)):
                return self._type_err(val, "float")
        elif expected_type.type == "bool":
            if not isinstance(val, bool):
                return self._type_err(val, "bool")
        elif expected_type.type == "string":
            if not isinstance(val, (str, unicode)):
                return self._type_err(val, "string")
        else:
            return self.get(expected_type.type).validate(val)
        return True, None

    def _type_err(self, val, expected):
        return False, "'%s' is of type %s, expected %s" % (val, type(val), expected)

class Interface(object):
    """
    Represents a Barrister IDL 'interface' entity.
    """

    def __init__(self, iface, contract):
        """
        Creates an Interface. Creates a 'functions' list of Function objects for
        each function defined on the interface.

        :Parameters:
          iface
            Dict representing the interface (from parsed IDL)
          contract
            Contract instance to associate the interface instance with
        """
        self.name = iface["name"]
        self.functions = { }
        for f in iface["functions"]:
            self.functions[f["name"]] = Function(self.name, f, contract)

    def function(self, func_name):
        """
        Returns the Function instance associated with the given func_name, or raises a
        RpcException if no function matches.
        """
        if self.functions.has_key(func_name):
            return self.functions[func_name]
        else:
            raise RpcException(ERR_METHOD_NOT_FOUND, 
                               "%s: Unknown function: '%s'", self.name, func_name)

class Enum(object):
    """
    Represents a Barrister IDL 'enum' entity.
    """

    def __init__(self, enum):
        """
        Creates an Enum.

        :Parameters:
          enum
            Dict representing the enum (from parsed IDL)
        """
        self.name = enum["name"]
        self.values = [ ]
        for v in enum["values"]:
            self.values.append(v["value"])

    def validate(self, val):
        """
        Validates that the val is in the list of values for this Enum.

        Returns two element tuple: (bool, string)

        - `bool` - True if valid, False if not
        - `string` - Description of validation error, or None if valid

        :Parameters:
          val
            Value to validate.  Should be a string.
        """
        if val in self.values:
            return True, None
        else:
            return False, "'%s' is not in enum: %s" % (val, str(self.values))

class Struct(object):
    """
    Represents a Barrister IDL 'struct' entity.
    """

    def __init__(self, s, contract):
        """
        Creates a Struct.

        :Parameters:
          s
            Dict representing the struct (from parsed IDL)
          contract
            Contract instance to associate with the Struct
        """
        self.contract = contract
        self.name = s["name"]
        self.extends = s["extends"]
        self.parent = None
        self.fields = { }
        for f in s["fields"]:
            self.fields[f["name"]] = Type(f)

    def field(self, name):
        """
        Returns the field on this struct with the given name. Will try to find this 
        name on all ancestors if this struct extends another.

        If found, returns a dict with keys: 'name', 'comment', 'type', 'is_array'
        If not found, returns None

        :Parameters:
          name
            string name of field to lookup
        """
        if self.fields.has_key(name):
            return self.fields[name]
        elif self.extends:
            if not self.parent:
                self.parent = self.contract.struct(self.extends)
            return self.parent.field(name)
        else:
            return None

    def validate(self, val):
        """
        Validates that the val matches the expected fields for this struct.
        val must be a dict, and must contain only fields represented by this struct and its
        ancestors.

        Returns two element tuple: (bool, string)

        - `bool` - True if valid, False if not
        - `string` - Description of validation error, or None if valid

        :Parameters:
          val
            Value to validate.  Must be a dict
        """
        if type(val) is not dict:
            return False, "%s is not a dict" % (str(val))

        for k, v in val.items():
            field = self.field(k)
            if field:
                ok, msg = self.contract.validate(field, field.is_array, v)
                if not ok:
                    return False, "field '%s': %s" % (field.name, msg)
            else:
                return False, "field '%s' not found in struct %s" % (k, self.name)

        all_fields = self.get_all_fields([])
        for field in all_fields:
            if not val.has_key(field.name) and not field.optional:
                return False, "field '%s' missing from: %s" % (field.name, str(val))

        return True, None

    def get_all_fields(self, arr):
        """
        Returns a list containing this struct's fields and all the fields of
        its ancestors.  Used during validation.
        """
        for k, v in self.fields.items():
            arr.append(v)
            
        if self.extends:
            parent = self.contract.get(self.extends)
            if parent:
                return parent.get_all_fields(arr)

        return arr

class Function(object):
    """
    Represents a function defined on an Interface
    """

    def __init__(self, iface_name, f, contract):
        """
        Creates a new Function

        :Parameters:
          iface_name
            Name of interface this function belongs to
          f
            Dict from parsed IDL representing this function. keys: 'name', 'params', 'returns'
          contract
            Contract to associate this Function with
        """
        self.contract = contract
        self.name = f["name"]
        self.params = []
        for p in f["params"]:
            self.params.append(Type(p))
        self.returns = Type(f["returns"])
        self.full_name = "%s.%s" % (iface_name, self.name)
        
    def validate_params(self, params):
        """
        Validates params against expected types for this function.  
        Raises RpcException if the params are invalid.
        """
        plen = 0
        if params != None:
            plen = len(params)

        if len(self.params) != plen:
            vals = (self.full_name, len(self.params), plen)
            msg = "Function '%s' expects %d param(s). %d given." % vals
            raise RpcException(ERR_INVALID_PARAMS, msg)
        
        if params != None:
            i = 0
            for p in self.params:
                self._validate_param(p, params[i])
                i += 1

    def validate_response(self, resp):
        """
        Validates resp against expected return type for this function.  
        Raises RpcException if the response is invalid.
        """
        ok, msg = self.contract.validate(self.returns, 
                                         self.returns.is_array, resp)
        if not ok:
            vals = (self.full_name, str(resp), msg)
            msg = "Function '%s' invalid response: '%s'. %s" % vals
            raise RpcException(ERR_INVALID_RESP, msg)

    def _validate_param(self, expected, param):
        """
        Validates a single param against its expected type.
        Raises RpcException if the param is invalid
        
        :Parameters:
          expected
            Type instance
          param
            Parameter value to validate
        """
        ok, msg = self.contract.validate(expected, expected.is_array, param)
        if not ok:
            vals = (self.full_name, expected.name, msg)
            msg = "Function '%s' invalid param '%s'. %s" % vals
            raise RpcException(ERR_INVALID_PARAMS, msg)

class Type(object):

    def __init__(self, type_dict):
        self.name = ""
        self.optional = False
        if type_dict.has_key("name"):
            self.name = type_dict["name"]
        self.type = type_dict["type"]
        self.is_array = type_dict["is_array"]
        if type_dict.has_key("optional"):
            self.optional = type_dict["optional"]
